@@ -9,6 +9,7 @@
 (use-modules (nyacc lang c99 munge))
 (use-modules (nyacc lang c99 util1))
 (use-modules (nyacc lang util))
+(use-modules (nyacc lang sx-match))
 (use-modules (nyacc lex))
 (use-modules (nyacc util))
 (use-modules (ice-9 pretty-print))
@@ -17,18 +18,19 @@
 (define cpp-defs
   (cond
    ((string-contains %host-type "darwin")
-    '("__GNUC__=6")
-    (remove (lambda (s)
-	      (string-contains s "_ENVIRONMENT_MAC_OS_X_VERSION"))
-	    (get-gcc-cpp-defs)))
+    (append
+     '("__GNUC__=6" "__signed=signed")
+     #;(remove (lambda (s)
+	       (string-contains s "_ENVIRONMENT_MAC_OS_X_VERSION"))
+	     (get-gcc-cpp-defs))
+     ))
    (else
     '())))
 (define inc-dirs
   (append
    `(,(assq-ref %guile-build-info 'includedir)
-     "/opt/local/include/glib-2.0"
-     "/opt/local/lib/glib-2.0/include"
-     "/usr/include")
+     "/usr/include"
+     "c99-exam")
    (get-gcc-inc-dirs)))
 (define inc-help
   (cond
@@ -51,9 +53,9 @@
        )
       ))))
 
-(define mode 'file)
 (define mode 'decl)
 (define mode 'code)
+(define mode 'file)
 (define debug #f)
 
 (define (parse-file file)
@@ -79,6 +81,7 @@
   (parse-string (apply string-append str-l)))
 
 (define (sf fmt . args) (apply simple-format #t fmt args))
+(define pp pretty-print)
 (define ppsx (lambda (sx) (pretty-print sx #:per-line-prefix "  ")))
 (define pp99 pretty-print-c99)
 
@@ -110,49 +113,48 @@
 	      "} foo_t;\n"
 	      ))
        (code "double x = 123.4f;")
-       (code "#if L'c'\nint x = 1;\n#endif\n")
+       ;;(code "#if L'c'\nint x = 1;\n#endif\n")
+       ;;(code "#include <Eina.h>\nint x = 1;\n")
+       ;;(code "wchar_t x = U'c';\n")
+       (code "char x = '\\b';\nchar *y = \"\\b\";\n")
+       (code (string-append
+	      "typedef int *bla_t[2];\n"
+	      "bla_t foo(bla_t (*)(bla_t));\n"))
        (indx 2)
+       (tree (parse-c99x "\"\""))
        ;;(tree (parse-string code))
-       (tree (parse-file "xxx.c"))
+       ;;(tree (parse-file "xxx.c"))
        ;;(udict (c99-trans-unit->udict tree))
+       ;;(decl (and=> ((sxpath `((decl ,indx))) tree) car))
+       ;;(xdecl (expand-typerefs decl udict))
        ;;(ddict (udict-enums->ddict udict))
-       ;;(udecl (udict-ref udict "z"))
+       ;;(udecl (udict-ref udict "x"))
+       ;;(mdecl (udecl->mspec udecl))
        ;;(decl (and=> ((sxpath `((decl ,indx))) tree) car))
        ;;(xdecl (expand-typerefs udecl udict '((struct . "foo"))))
        )
   ;;(display code)
-  (ppsx tree)
-  ;;(pp99 tree)
-  ;;(ppsx xdecl)
-  ;;(display "==\n")
+  ;;(ppsx tree)
+  ;;(ppsx decl)
   ;;(pp99 xdecl)
-  ;;(ppsx mspec)
-  ;;(ppsx (expand-typerefs udecl udict '("foo_t")))
-  ;;(ppsx ddict)
-  ;;(set! adecl decl)
+  ;;(ppsx (get-gcc-inc-dirs))
   #t)
 
+#|
+(sf "~S\n"
+    (with-input-from-string "'\\177'"
+	(lambda () (read-c-chlit (read-char)))))
+
+(sf "~S\n"
+    (with-input-from-string "\"\\177\""
+	(lambda () (read-c-string (read-char)))))
+|#
+
+;;(pp (sx-match '(foo (bar "baz") "hello")
+;;       ((foo (bar . ,text) . ,rest) #t) (* #f)))
+
 ;; ex12.c: illustrates removal of comment prefix, offset-8 => offset-2
-;;(let ((tree (parse-file "exam.d/ex12.c"))) (pp99 tree))
-      
+
+;;(let ((tree (parse-file "c99-exam/ex01.c"))) (pp99 tree))
 ;;(ugly-print (quote `(abc ,@def)))
-
-(define n1570a ;; sec 6.7.8
-  (string-append
-   "typedef signed int t;\n"
-   "typedef int plain;\n"
-   "struct tag {\n"
-   "  unsigned t:4;\n"
-   "  const t:5;\n"
-   "  plain r:5;\n"
-   "};\n"))
-(define n1570b
-  (string-append
-   n1570a
-   "int foo() {\n"
-   "  t f(t (t));\n"
-   "  long t;\n"
-   "  return 1;\n"
-   "};\n"))
-
 ;; --- last line ---
