@@ -1,19 +1,19 @@
 ;;; nyacc/lang/matlab/util.scm - matlab processing code
-;;; 
-;;; Copyright (C) 2016 Matthew R. Wette
-;;;
-;;; This program is free software: you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by 
-;;; the Free Software Foundation, either version 3 of the License, or 
-;;; (at your option) any later version.
-;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
-;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+
+;; Copyright (C) 2016,2018 Matthew R. Wette
+;;
+;; This library is free software; you can redistribute it and/or
+;; modify it under the terms of the GNU Lesser General Public
+;; License as published by the Free Software Foundation; either
+;; version 3 of the License, or (at your option) any later version.
+;;
+;; This library is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;; Lesser General Public License for more details.
+;;
+;; You should have received a copy of the GNU Lesser General Public License
+;; along with this library; if not, see <http://www.gnu.org/licenses/>.
 
 ;; utilities for processing output trees
 
@@ -24,9 +24,10 @@
 	    name-expr->decl
 	    )
   #:use-module (nyacc lang util)
+  #:use-module (nyacc lang sx-match)
   #:use-module (ice-9 pretty-print)
+  #:use-module (ice-9 regex)
   #:use-module ((sxml fold) #:select (foldts*-values))
-  #:use-module (sxml match)
   )
 
 ;; probably also need some sort of overall declaration form
@@ -53,7 +54,7 @@
 ;; decl
 ;; decl: fctn, struct, array, double, int
 #;(define (name-expr->c-decl name expr) ;; => decl
-  (sxml-match expr
+  (sx-match expr
     ((aref-or-call (ident "zeros") ,ex-l)
      #f)))
 
@@ -90,7 +91,7 @@
 ;; Given an lval return the root identifier name as a string.
 ;; @var{disp} is the disposition (e.g., struct) of the lval
 (define* (lval->ident lval #:optional (disp 'unknown))
-  (sxml-match lval
+  (sx-match lval
     ((ident ,name) (cons name disp))
     ((sel ,ident ,lval) (lval->ident lval 'struct))
     ((array-ref ,lval ,ex-l)
@@ -101,8 +102,7 @@
      (case disp
        ((struct) (lval->ident lval disp))
        (else (lval->ident lval 'array))))
-    (,otherwise
-     (throw 'util-error "unknown lval: ~S" lval))))
+    (* (throw 'util-error "unknown lval: ~S" lval))))
 
 (define (binary-rank lval rval)
   (and lval rval (max lval rval)))
@@ -133,12 +133,12 @@
 ;; should be (for now) struct double vector matrix
 
 (define fold-in-decl
-  (let ((rx1 (make-regexp "^~\s*([0-9A-Za-z]+)\s*:\s*([0-9A-Za-z]+)")))
+  (let ((rx1 (make-regexp "^~\\s*([0-9A-Za-z]+)\\s*:\\s*([0-9A-Za-z]+)")))
     (lambda (str seed)
-      (let ((m (regexp-exec str)))
+      (let ((m (regexp-exec rx1 str)))
 	(if m
-	    (acons (match:string m 1) (match:string m 2) seed)
-	    seeed)))))
+	    (acons (match:substring m 1) (match:substring m 2) seed)
+	    seed)))))
 
 ;; matrix: oset rstr cstr flag double *
 ;; vector: double *
@@ -162,7 +162,7 @@
   ;; Then pass through c99 parser OR Maybe not
 
   (define (fD tree seed dict) ;; => (values tree seed dict)
-    (sxml-match tree
+    (sx-match tree
       ((function-file (@ (file ,name)) . ,rest)
        (values tree '()
 	       (cons*
@@ -188,8 +188,7 @@
        ;;(values tree '() (d-add-rank dict name (length (sx-tail lval 1)))))
        (values tree '() dict))
 
-      (,otherwise
-       (values tree '() dict))))
+      (* (values tree '() dict))))
 
   (define (fU tree seed dict kseed kdict) ;; => (values seed dict)
     ;;(when (pair? tree) (simple-format #t "cartree=~S\n" (car tree)))
@@ -230,7 +229,7 @@
 (define (declify-script tree . rest)
   
   (define (fD tree seed dict) ;; => (values tree seed dict)
-    (sxml-match tree
+    (sx-match tree
       ((script-file (@ (file ,name)) . ,rest)
        (values tree '()
 	       (cons*
@@ -255,8 +254,7 @@
        ;;(fout "    ->rank =>~S\n" (expr->rank lval))
        (values tree '() dict))
 
-      (,otherwise
-       (values tree '() dict))))
+      (* (values tree '() dict))))
   
   (define (fU tree seed dict kseed kdict) ;; => (values seed dict)
     ;;(fout "tree-tag=~S kseed=~S\n" (car tree) kseed)
@@ -265,7 +263,7 @@
       ((assn)
        (let* ((assn (reverse kseed))
 	      (lval (sx-ref assn 1)) (rval (sx-ref assn 2))
-	      (ltyp (sx-atr-ref lval 'type))
+	      (ltyp (sx-attr-ref lval 'type))
 	      (rtype (sx-attr-ref rval 'type))
 	      (rrank (and=> (sx-attr-ref rval 'rank) string->number))
 	      )
