@@ -12,9 +12,8 @@
 ;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 ;;; Lesser General Public License for more details.
 ;;;
-;;; You should have received a copy of the GNU Lesser General Public
-;;; License along with this library; if not, write to the Free Software
-;;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+;;; You should have received a copy of the GNU Lesser General Public License
+;;; along with this library; if not, see <http://www.gnu.org/licenses/>
 
 ;; Convert guile lalr grammar to nyacc grammar.
 
@@ -25,6 +24,10 @@
   #:export (lalr-parser guile-lalr->nyacc-lalr)
   #:use-module ((srfi srfi-1) #:select (fold-right))
   )
+(cond-expand
+ (guile-2 #t)
+ (else
+  (use-modules (ice-9 syncase))))
 
 (define (convert-tree spec0)
   (let* ((terms (cons '*eoi* (car spec0)))
