@@ -1,19 +1,19 @@
-;; lang/c99/body.scm
-;;
-;; Copyright (C) 2015-2017 Matthew R. Wette
-;;
-;; This program is free software: you can redistribute it and/or modify
-;; it under the terms of the GNU General Public License as published by 
-;; the Free Software Foundation, either version 3 of the License, or 
-;; (at your option) any later version.
-;;
-;; This program is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;; GNU General Public License for more details.
-;;
-;; You should have received a copy of the GNU General Public License
-;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;;; lang/c99/body.scm
+;;;
+;;; Copyright (C) 2015-2018 Matthew R. Wette
+;;;
+;;; This library is free software; you can redistribute it and/or
+;;; modify it under the terms of the GNU Lesser General Public
+;;; License as published by the Free Software Foundation; either
+;;; version 3 of the License, or (at your option) any later version.
+;;;
+;;; This library is distributed in the hope that it will be useful,
+;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;;; Lesser General Public License for more details.
+;;;
+;;; You should have received a copy of the GNU Lesser General Public License
+;;; along with this library; if not, see <http://www.gnu.org/licenses/>.
 
 ;; Notes on the code design may be found in doc/nyacc/lang/c99-hg.info
 
@@ -29,9 +29,9 @@
 
 ;; issue w/ brlev: not intended to beused with `extern "C" {'
 
+(use-modules ((srfi srfi-1) #:select (fold-right)))
 (use-modules ((srfi srfi-9) #:select (define-record-type)))
-(use-modules ((sxml xpath) #:select (sxpath)))
-(use-modules (ice-9 pretty-print)) ;; for debugging
+(use-modules (ice-9 pretty-print))	; for debugging
 
 (define-record-type cpi
   (make-cpi-1)
@@ -110,12 +110,9 @@
 	   (lambda (tl) (set-cpi-ctl! cpi (append tl (cpi-ctl cpi)))))
     (and=> (assoc-ref (cpi-idefd cpi) "__builtin")
 	   (lambda (tl) (set-cpi-defs! cpi (append tl (cpi-defs cpi)))))
-    ;;(sferr "cpi-defs=\n")
-    ;;(pretty-print (cpi-defs cpi) (current-error-port))
-    ;; Return the populated info.
     cpi))
 
-(define *info* (make-fluid #f))
+(define *info* (make-fluid))
 	  
 ;; @deffn {Procedure} typename? name
 ;; Called by lexer to determine if symbol is a typename.
@@ -131,16 +128,16 @@
 
 (define cpi-inc-blev!
   (case-lambda
-   ((info) (set-cpi-blev! info (1+ (cpi-blev info))))
-   (() (cpi-inc-blev! (fluid-ref *info*)))))
+    ((info) (set-cpi-blev! info (1+ (cpi-blev info))))
+    (() (cpi-inc-blev! (fluid-ref *info*)))))
 (define cpi-dec-blev!
   (case-lambda
-   ((info) (set-cpi-blev! info (1- (cpi-blev info))))
-   (() (cpi-dec-blev! (fluid-ref *info*)))))
+    ((info) (set-cpi-blev! info (1- (cpi-blev info))))
+    (() (cpi-dec-blev! (fluid-ref *info*)))))
 (define cpi-top-blev?
   (case-lambda
-   ((info) (zero? (cpi-blev info)))
-   (() (cpi-top-blev? (fluid-ref *info*)))))
+    ((info) (zero? (cpi-blev info)))
+    (() (cpi-top-blev? (fluid-ref *info*)))))
 
 ;; @deffn {Procedure} add-typename name
 ;; Helper for @code{save-typenames}.
@@ -213,7 +210,8 @@
   (if (not (eq? ch #\#)) #f
       (let iter ((cl '()) (ch (read-char)))
 	(cond
-	 ((eof-object? ch) (throw 'cpp-error "CPP lines must end in newline"))
+	 ;;((eof-object? ch) (throw 'cpp-error "CPP lines must end in newline"))
+	 ((eof-object? ch) (list->string (reverse cl)))
 	 ((eq? ch #\newline) (unread-char ch) (list->string (reverse cl)))
 	 ((eq? ch #\\)
 	  (let ((c2 (read-char)))
@@ -284,11 +282,17 @@
 	 ;;
 	 (t-ident (assq-ref symtab '$ident))
 	 (t-typename (assq-ref symtab 'typename))
-	 (xp1 (sxpath '(cpp-stmt define)))
-	 (xp2 (sxpath '(decl))))
+	 (xp1 (lambda (stmts)
+		(fold-right
+		 (lambda (stmt seed)
+		   (if (and (eqv? 'cpp-stmt (sx-tag stmt))
+			    (eqv? 'define (sx-tag (sx-ref stmt 1))))
+		       (cons (sx-ref stmt 1) seed)
+		       seed))
+		 '() stmts))))
     ;; mode: 'code|'file|'decl
     ;; xdef?: (proc name mode) => #t|#f  : do we expand #define?
-    (lambda* (#:key (mode 'code) (xdef? #f))
+    (lambda* (#:key (mode 'code) xdef? show-includes)
       (let ((bol #t)		 ; begin-of-line condition
 	    (suppress #f)	 ; parsing cpp expanded text (kludge?)
 	    (ppxs (list 'keep))	 ; CPP execution state stack
@@ -312,22 +316,24 @@
 	  
 	  (define (apply-helper file)
 	    ;; file will include <> or "", need to strip
-	    (let* ((file (substring/shared file 1 (1- (string-length file))))
-		   (tyns (assoc-ref (cpi-itynd info) file))
+	    (let* ((tyns (assoc-ref (cpi-itynd info) file))
 		   (defs (assoc-ref (cpi-idefd info) file)))
 	      (when tyns
 		(for-each add-typename tyns)
 		(set-cpi-defs! info (append defs (cpi-defs info))))
 	      tyns))
 
-	  (define (inc-stmt->file stmt) ;; retain <> or ""
+	  (define (inc-stmt->file-spec stmt) ;; retain <> or ""
 	    (let* ((arg (cadr stmt)))
 	      (if (ident-like? arg) ;; #include MYFILE
 		  (expand-cpp-macro-ref arg (cpi-defs info))
 		  arg)))
 
-	  (define (inc-file->path file next)
-	    (find-incl-in-dirl file (cpi-incs info) next))
+	  (define (file-spec->file spec)
+	    (substring/shared spec 1 (1- (string-length spec))))
+
+	  (define (inc-file-spec->path spec next)
+	    (find-incl-in-dirl spec (cpi-incs info) next))
 
 	  (define (code-if stmt)
 	    (case (car ppxs)
@@ -369,9 +375,10 @@
 	    stmt)
 	  
 	  (define* (eval-cpp-incl/here stmt #:optional next) ;; => stmt
-	    (let* ((file (inc-stmt->file stmt))
-		   (path (inc-file->path file next)))
-	      ;;(sferr "include ~S => ~S\n" file path)
+	    (let* ((spec (inc-stmt->file-spec stmt))
+		   (file (file-spec->file spec))
+		   (path (inc-file-spec->path spec next)))
+	      (if show-includes (sferr "include ~S => ~S\n" file path))
 	      (cond
 	       ((apply-helper file))
 	       ((not path) (c99-err "not found: ~S" file)) ; file not found
@@ -380,9 +387,10 @@
 
 	  (define* (eval-cpp-incl/tree stmt #:optional next) ;; => stmt
 	    ;; include file as a new tree
-	    (let* ((file (inc-stmt->file stmt))
-		   (path (inc-file->path file next)))
-	      ;;(sferr "include ~S\n" path)
+	    (let* ((spec (inc-stmt->file-spec stmt))
+		   (file (file-spec->file spec))
+		   (path (inc-file-spec->path spec next)))
+	      (if show-includes (sferr "include ~S => ~S\n" file path))
 	      (cond
 	       ((apply-helper file) stmt)		 ; use helper
 	       ((not path) (c99-err "not found: ~S" file)) ; file not found
@@ -508,6 +516,7 @@
 			  (assc-$ `(cpp-stmt . ,stmt))
 			  (iter (read-char))))))
 		 (else (iter ch))))
+	       ((read-c-chlit ch) => assc-$) ; before ident for [ULl]'c'
 	       ((read-c-ident ch) =>
 		(lambda (name)
 		  (let ((symb (string->symbol name))
@@ -528,7 +537,6 @@
 		      (cons (assq-ref symtab '$ident) name))))))
 	       ((read-c-num ch) => assc-$)
 	       ((read-c-string ch) => assc-$)
-	       ((read-c-chlit ch) => assc-$)
 	       ((read-c-comm ch #f #:skip-prefix #t) => assc-$)
 	       ((and (char=? ch #\{)	; ugly tracking of block-lev in lexer
 		     (eqv? 'keep (car ppxs)) (cpi-inc-blev! info) #f) #f)
