@@ -1,19 +1,19 @@
-;; lang/c/cpp.scm
-;;
-;; Copyright (C) 2015-2017 Matthew R. Wette
-;;
-;; This program is free software: you can redistribute it and/or modify
-;; it under the terms of the GNU General Public License as published by 
-;; the Free Software Foundation, either version 3 of the License, or 
-;; (at your option) any later version.
-;;
-;; This program is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;; GNU General Public License for more details.
-;;
-;; You should have received a copy of the GNU General Public License
-;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;;; lang/c/cpp.scm
+;;;
+;;; Copyright (C) 2015-2018 Matthew R. Wette
+;;;
+;;; This library is free software; you can redistribute it and/or
+;;; modify it under the terms of the GNU Lesser General Public
+;;; License as published by the Free Software Foundation; either
+;;; version 3 of the License, or (at your option) any later version.
+;;;
+;;; This library is distributed in the hope that it will be useful,
+;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;;; Lesser General Public License for more details.
+;;;
+;;; You should have received a copy of the GNU Lesser General Public License
+;;; along with this library; if not, see <http://www.gnu.org/licenses/>.
 
 ;; C preprocessor macro expansion and condition text parse-and-eval
 ;; ref: https://gcc.gnu.org/onlinedocs/gcc-3.0.1/cpp_3.html
@@ -29,12 +29,14 @@
 	    eval-cpp-expr)
   #:use-module (nyacc parse)
   #:use-module (nyacc lex)
-  #:use-module (nyacc lang util)
-  #:use-module (rnrs arithmetic bitwise)
-  #:use-module (ice-9 match)
-  #:use-module (ice-9 textual-ports)
-  #:use-module (system base pmatch)
-  )
+  #:use-module (nyacc lang util))
+(cond-expand
+  (guile-2
+   (use-modules (rnrs arithmetic bitwise))
+   (use-modules (system base pmatch)))
+  (else
+   (use-modules (ice-9 optargs))
+   (use-modules (nyacc compat18))))
 
 (define c99-std-defs
   '("__DATE__" "__FILE__" "__LINE__" "__STDC__" "__STDC_HOSTED__"
@@ -69,6 +71,13 @@
    ((char-set-contains? inline-whitespace ch) (skip-il-ws (read-char)))
    (else ch)))
 
+;; This reads the rest of the input, with ch and returns a string;
+;; Replaces get-string-all from (ice-9 textual-ports).
+(define (read-rest ch)
+  (list->string (let iter ((ch ch))
+		  (if (eof-object? ch) '()
+		      (cons ch (iter (read-char)))))))
+
 ;; Not sure about this. We want to turn a list of tokens into a string
 ;; with proper escapes.
 (define (esc-c-str str)
@@ -136,9 +145,7 @@
 	   ((eq? la #\,) (iter args (skip-il-ws (read-char))))))
 	(begin (if (char? la) (unread-char la)) #f)))
 
-  (define (p-rest la) ;; parse rest
-    (cond ((eof-object? la) "")
-	  (else (unread-char la) (get-string-all (current-input-port)))))
+  (define (p-rest la) (read-rest la))
 
   (let* ((name (read-c-ident (skip-il-ws (read-char))))
 	 (args (or (p-args (read-char)) '()))
@@ -174,9 +181,7 @@
 (define (cpp-line->stmt line)
   (define (rd-ident) (read-c-ident (skip-il-ws (read-char))))
   (define (rd-num) (and=> (read-c-num (skip-il-ws (read-char))) cdr))
-  (define (rd-rest) (let ((ch (skip-il-ws (read-char))))
-		      (if (not (eof-object? ch)) (unread-char ch))
-		      (get-string-all (current-input-port))))
+  (define (rd-rest) (read-rest (skip-il-ws (read-char))))
   (with-input-from-string line
     (lambda ()
       (let ((ch (skip-il-ws (read-char))))
@@ -252,7 +257,7 @@
 (define* (eval-cpp-expr tree #:optional (dict '()) #:key (inc-dirs '()))
   (letrec
       ((tx (lambda (tr ix) (sx-ref tr ix)))
-       (tx1 (lambda (tr) (tx tr 1)))
+       (tx1 (lambda (tr) (sx-ref tr 1)))
        (ev (lambda (ex ix) (eval-expr (sx-ref ex ix))))
        (ev1 (lambda (ex) (ev ex 1)))	; eval expr in arg 1
        (ev2 (lambda (ex) (ev ex 2)))	; eval expr in arg 2
@@ -459,7 +464,9 @@
 ;; one @code{#\space} is re-inserted.
 ;; @end deffn
 (define (collect-args argl defs used)
+  ;;(sferr "collect-args\n")
   (let iter1 ((sp #f) (ch (read-char)))
+    ;;(sferr "collect-args iter: ch=~S\n" ch)
     (cond
      ((eof-object? ch) (if sp (unread-char #\space)) #f)
      ((char-set-contains? inline-whitespace ch) (iter1 #t (read-char)))
@@ -574,6 +581,9 @@
 ;; ("ABC" . "123")
 ;; ("MAX" ("X" "Y") . "((X)>(Y)?(X):(Y))")
 ;; @end example
+;; @noindent
+;; Note that this routine will look in the current-input so if you want to
+;; expand text, 
 ;; @end deffn
 (define* (expand-cpp-macro-ref ident defs #:optional (used '()))
   (let ((rval (assoc-ref defs ident)))
@@ -586,6 +596,9 @@
 	    (or (expand-cpp-macro-ref repl defs used) repl)
 	    repl)))
      ((pair? rval)
+      ;; GNU CPP manual: "A function-like macro is only expanded if its name
+      ;; appears with a pair of parentheses after it.  If you just write the
+      ;; name, it is left alone."
       (and=> (collect-args (car rval) defs used)
 	     (lambda (argd)
 	       (let* ((used (cons ident used))
