@@ -1,19 +1,19 @@
 ;;; lang/c99/cppmach.scm
 ;;;
-;;; Copyright (C) 2015,2016 Matthew R. Wette
+;;; Copyright (C) 2015,2016,2018 Matthew R. Wette
 ;;;
-;;; This program is free software: you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by 
-;;; the Free Software Foundation, either version 3 of the License, or 
-;;; (at your option) any later version.
+;;; This library is free software; you can redistribute it and/or
+;;; modify it under the terms of the GNU Lesser General Public
+;;; License as published by the Free Software Foundation; either
+;;; version 3 of the License, or (at your option) any later version.
 ;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
+;;; This library is distributed in the hope that it will be useful,
+;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;;; Lesser General Public License for more details.
 ;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;;; You should have received a copy of the GNU Lesser General Public License
+;;; along with this library; if not, see <http://www.gnu.org/licenses/>.
 
 ;; C preprocessor expression parser generator
 
@@ -93,8 +93,11 @@
      (postfix-expression "--" ($$ `(post-dec ,$1))))
     (primary-expression
      ($ident ($$ `(ident ,$1)))
-     ($fixed ($$ `(fixed ,$1)))	; integer-constant
-     ($chlit ($$ `(char ,$1)))	; char-constant
+     ($fixed ($$ `(fixed ,$1)))		; integer literal
+     ($chlit ($$ `(char ,$1)))		; char literal
+     ($chlit/L ($$ `(char (@ (type "wchar_t")) ,$1)))
+     ($chlit/u ($$ `(char (@ (type "char16_t")) ,$1)))
+     ($chlit/U ($$ `(char (@ (type "char32_t")) ,$1)))
      ("defined" "(" $ident ")" ($$ `(defined ,$3)))
      ("defined" $ident ($$ `(defined ,$2)))
      ("__has_include__" "(" $string ")" ($$ `(has-include ,$3)))
