@@ -1,19 +1,19 @@
-;; lang/c99/mach.scm
-;;
-;; Copyright (C) 2015-2017 Matthew R. Wette
-;;
-;; This program is free software: you can redistribute it and/or modify
-;; it under the terms of the GNU General Public License as published by 
-;; the Free Software Foundation, either version 3 of the License, or 
-;; (at your option) any later version.
-;;
-;; This program is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;; GNU General Public License for more details.
-;;
-;; You should have received a copy of the GNU General Public License
-;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;;; lang/c99/mach.scm
+;;;
+;;; Copyright (C) 2015-2018 Matthew R. Wette
+;;;
+;;; This library is free software; you can redistribute it and/or
+;;; modify it under the terms of the GNU Lesser General Public
+;;; License as published by the Free Software Foundation; either
+;;; version 3 of the License, or (at your option) any later version.
+;;;
+;;; This library is distributed in the hope that it will be useful,
+;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;;; Lesser General Public License for more details.
+;;;
+;;; You should have received a copy of the GNU Lesser General Public License
+;;; along with this library; if not, see <http://www.gnu.org/licenses/>.
 
 ;; C parser generator: based on ISO-C99; with comments and CPP statements
 
@@ -323,6 +323,9 @@
      (lone-comment ($$ (make-tl 'field-list $1)))
      (struct-declaration-list struct-declaration ($$ (tl-append $1 $2)))
      (struct-declaration-list lone-comment ($$ (tl-append $1 $2)))
+     ;; the following added 13 Nov 2017
+     (";" ($$ (make-tl 'field-list)))
+     (struct-declaration-list ";" ($$ $1))
      )
 
     (struct-declaration			; S 6.7.2.1
@@ -709,9 +712,13 @@
      ('cpp-ident ($$ `(ident ,$1)))
      )
     (constant
-     ($fixed ($$ `(fixed ,$1)))		; integer-constant
-     ($float ($$ `(float ,$1)))		; floating-constant
-     ($chlit ($$ `(char ,$1))))		; char-constant
+     ($fixed ($$ `(fixed ,$1)))		; integer literal
+     ($float ($$ `(float ,$1)))		; floating literal
+     ($chlit ($$ `(char ,$1)))		; char literal
+     ($chlit/L ($$ `(char (@ (type "wchar_t")) ,$1)))
+     ($chlit/u ($$ `(char (@ (type "char16_t")) ,$1)))
+     ($chlit/U ($$ `(char (@ (type "char32_t")) ,$1)))
+     )
     (string-literal
      ($string ($$ (make-tl 'string $1))) ; string-constant
      (string-literal $string ($$ (tl-append $1 $2))))
