@@ -1,19 +1,19 @@
 ;;; nyacc/lang/c99/munge.scm - C processing code
 ;;; 
-;;; Copyright (C) 2015-2017 Matthew R. Wette
+;;; Copyright (C) 2015-2018 Matthew R. Wette
 ;;;
-;;; This program is free software: you can redistribute it and/or modify
-;;; it under the terms of the GNU General Public License as published by 
-;;; the Free Software Foundation, either version 3 of the License, or 
-;;; (at your option) any later version.
+;;; This library is free software; you can redistribute it and/or
+;;; modify it under the terms of the GNU Lesser General Public
+;;; License as published by the Free Software Foundation; either
+;;; version 3 of the License, or (at your option) any later version.
 ;;;
-;;; This program is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
+;;; This library is distributed in the hope that it will be useful,
+;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;;; Lesser General Public License for more details.
 ;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;;; You should have received a copy of the GNU Lesser General Public License
+;;; along with this library; if not, see <http://www.gnu.org/licenses/>.
 
 ;; utilities for processing output trees
 
@@ -102,6 +102,7 @@
   #:use-module ((nyacc lang c99 cpp) #:select (eval-cpp-expr))
   #:use-module (nyacc lang c99 pprint)
   #:use-module (nyacc lang util)
+  #:use-module (nyacc lang sx-match)
   #:use-module ((sxml fold) #:select (foldts foldts*))
   #:use-module (sxml match)
   #:use-module (srfi srfi-11)		; let-values
@@ -149,23 +150,24 @@
      (else #f))))
 (define (pointer-declr? declr)
   ;;(sferr "pointer-declr? ~S\n" declr)
-  (sxml-match declr
-    (#f #f)
-    ((init-declr ,declr) (pointer-declr? declr))
-    ((comp-declr ,declr) (pointer-declr? declr))
-    ((param-declr ,declr) (pointer-declr? declr))
-    ;;
-    ((ptr-declr ,pointer ,dir-declr) #t)
-    ((array-of . ,rest) #t)
-    ((ftn-declr . ,rest) #t)
-    ((abs-declr (pointer . ,r1) . ,r2) #t)
-    ;;
-    ((init-declr-list ,declrs ...)
-     (fold (lambda (dcl seed) (and (pointer-declr? dcl) seed)) #t declrs))
-    ((comp-declr-list ,declrs ...)
-     (fold (lambda (dcl seed) (and (pointer-declr? dcl) seed)) #t declrs))
-    ;;
-    (,otherwise #f)))
+  (and
+   declr
+   (sx-match declr
+     ((init-declr ,declr) (pointer-declr? declr))
+     ((comp-declr ,declr) (pointer-declr? declr))
+     ((param-declr ,declr) (pointer-declr? declr))
+     ;;
+     ((ptr-declr ,pointer ,dir-declr) #t)
+     ((array-of . ,rest) #t)
+     ((ftn-declr . ,rest) #t)
+     ((abs-declr (pointer . ,r1) . ,r2) #t)
+     ;;
+     ((init-declr-list . ,declrs)
+      (fold (lambda (dcl seed) (and (pointer-declr? dcl) seed)) #t declrs))
+     ((comp-declr-list . ,declrs)
+      (fold (lambda (dcl seed) (and (pointer-declr? dcl) seed)) #t declrs))
+     ;;
+     (* #f))))
 ;; @deffn {Procedure} pointer-stor-declr? declr => #t|#f
 ;; @deffnx {Procedure} pointer-pass-declr? declr => #t|#f
 ;; This predicate determines if the declarator is implemented as a pointer.
@@ -173,23 +175,24 @@
 ;; @end deffn
 ;;(define (pointer-stor-declr? declr)
 (define (pointer-pass-declr? declr)
-  (sxml-match declr
-    (#f #f)
-    ((init-declr ,declr) (pointer-declr? declr))
-    ((comp-declr ,declr) (pointer-declr? declr))
-    ((param-declr ,declr) (pointer-declr? declr))
-    ;;
-    ((ptr-declr ,pointer ,dir-declr) #t)
-    ((array-of . ,rest) #t)
-    ((ftn-declr . ,rest) #t)
-    ((abs-declr (pointer . ,r1) . ,r2) #t)
-    ;;
-    ((init-declr-list ,declrs ...)
-     (fold (lambda (dcl seed) (and (pointer-declr? dcl) seed)) #t declrs))
-    ((comp-declr-list ,declrs ...)
-     (fold (lambda (dcl seed) (and (pointer-declr? dcl) seed)) #t declrs))
-    ;;
-    (,otherwise #f)))
+  (and
+   declr
+   (sx-match declr
+     ((init-declr ,declr) (pointer-declr? declr))
+     ((comp-declr ,declr) (pointer-declr? declr))
+     ((param-declr ,declr) (pointer-declr? declr))
+     ;;
+     ((ptr-declr ,pointer ,dir-declr) #t)
+     ((array-of . ,rest) #t)
+     ((ftn-declr . ,rest) #t)
+     ((abs-declr (pointer . ,r1) . ,r2) #t)
+     ;;
+     ((init-declr-list . ,declrs)
+      (fold (lambda (dcl seed) (and (pointer-declr? dcl) seed)) #t declrs))
+     ((comp-declr-list . ,declrs)
+      (fold (lambda (dcl seed) (and (pointer-declr? dcl) seed)) #t declrs))
+     ;;
+     (* #f))))
 
 ;; Use the term @dfn{udecl}, or unit-declaration, for a declaration which has
 ;; only one decl-item.  That is where,
@@ -383,7 +386,7 @@
     (let*-values (((tag attr specl declrs tail) (split-decl decl))
 		  ((tag) (values 'udecl)))
       ;; TODO: for typedefs add attr (typedef "name") to associated udecls
-      (sxml-match specl
+      (sx-match specl
 
 	;; struct typedefs 
 	((decl-spec-list
@@ -429,8 +432,8 @@
 	((decl-spec-list
 	  (type-spec (struct-def (ident ,name) . ,rest2) . ,rest1))
 	 (update `(struct . ,name)
-		(make-udecl 'struct-def `((ident ,name) . ,rest2))
-		tag attr specl declrs tail seed))
+		 (make-udecl 'struct-def `((ident ,name) . ,rest2))
+		 tag attr specl declrs tail seed))
 	((decl-spec-list
 	  (type-spec (struct-def . ,rest2) . ,rest1))
 	 (iter-declrs tag attr specl declrs tail seed))
@@ -456,8 +459,7 @@
 	 (update `(enum . "*anon*") (make-udecl 'enum-def rest2)
 		 tag attr specl declrs tail seed))
 
-	(,otherwise
-	 (iter-declrs tag attr specl declrs tail seed)))))
+	(* (iter-declrs tag attr specl declrs tail seed)))))
    
    ((eqv? (sx-tag decl) 'comp-udecl) (acons (udecl-id decl) decl seed))
    ((eqv? (sx-tag decl) 'comp-decl) (unitize-comp-decl decl seed))
@@ -520,13 +522,13 @@
 	     (ident (declr-ident declr))
 	     (name (cadr ident)))
 	(acons name decl seed))))
-	
+
 ;; @deffn {Procedure} declr-ident declr => (ident "name")
 ;; Given a declarator, aka @code{init-declr}, return the identifier.
 ;; This is used by @code{trans-unit->udict}.
 ;; @end deffn
 (define (declr-ident declr)
-  (sxml-match declr
+  (sx-match declr
     ((ident ,name) declr)
     ((init-declr ,declr . ,rest) (declr-ident declr))
     ((comp-declr ,declr) (declr-ident declr))
@@ -534,10 +536,10 @@
     ((array-of ,dir-declr ,array-spec) (declr-ident dir-declr))
     ((array-of ,dir-declr) (declr-ident dir-declr))
     ((ptr-declr ,pointer ,dir-declr) (declr-ident dir-declr))
-    ((ftn-declr ,dir-declr ,rest ...) (declr-ident dir-declr))
+    ((ftn-declr ,dir-declr . ,rest) (declr-ident dir-declr))
     ((scope ,declr) (declr-ident declr))
     ((bit-field ,ident . ,rest) ident)
-    (,otherwise (throw 'util-error "c99/munge: unknown declarator: " declr))))
+    (* (throw 'util-error "c99/munge: unknown declarator: " declr))))
 
 ;; @deffn {Procedure} declr-id decl => "name"
 ;; This extracts the name from the return value of @code{declr-ident}.
@@ -601,9 +603,9 @@
 ;; @deffn {Procedure} typedef-decl? decl)
 ;; @end deffn
 (define (typedef-decl? decl)
-  (sxml-match decl
+  (sx-match decl
     ((decl (decl-spec-list (stor-spec (typedef)) . ,r1) . ,r2) #t)
-    (,otherwise #f)))
+    (* #f)))
 
 ;; @deffn {Procedure} repl-typespec decl-spec-list repl-type-spec
 ;; In the decl-spec-list replace the type-specifier.
@@ -690,7 +692,7 @@
 ;; @end example
 (define (tdef-splice-declr orig-declr tdef-declr)
   (define (probe-declr declr)
-    (sxml-match declr
+    (sx-match declr
       ((ident ,name)
        (sx-ref orig-declr 1))
       ((init-declr ,declr . ,rest)
@@ -709,7 +711,7 @@
        `(ftn-declr ,(probe-declr dir-declr) . ,rest))
       ((scope ,declr)
        `(scope ,(probe-declr declr)))
-      (,otherwise (throw 'util-error "c99/munge: unknown declarator: " declr))))
+      (* (throw 'util-error "c99/munge: unknown declarator: " declr))))
   (probe-declr tdef-declr))
 
 ;; @deffn {Procedure} tdef-splice-declr-list orig-declr-list tdef-declr
@@ -806,7 +808,7 @@
 	 (values fixd-specl declr)))
       ((struct-ref union-ref) ;; compound reference; replace unless pointer
        (let* ((c-name (and=> (sx-find 'ident tspec)
-				(lambda (id) (sx-ref id 1))))
+			     (lambda (id) (sx-ref id 1))))
 	      (c-key (compound-key class c-name)) ;; e.g., (struct . "foo")
 	      (c-decl (and c-key (assoc-ref udict c-key)))
 	      (t-spec (and c-decl (sx-find 'type-spec (sx-ref c-decl 1)))))
@@ -901,119 +903,120 @@
   ;; This will check for function declrs and fix parameters.
   (define (fix-declr declr)
     ;;(sferr "fix-declr:\n") (pperr declr)
-    (sxml-match declr
-      (#f declr)
-      ((ident ,name) declr)
-      ((bit-field . ,rest) declr)
-      
-      ((init-declr ,declr1 . ,rest)
-       (let ((xdeclr (fix-declr declr1)))
-	 (if (eq? xdeclr declr1) declr `(init-declr ,xdeclr . ,rest))))
-      ((comp-declr ,declr1)
-       (let ((xdeclr (fix-declr declr1)))
-	 (if (eq? xdeclr declr1) declr `(comp-declr ,xdeclr))))
-      ((param-declr ,declr1)
-       (let ((xdeclr (fix-declr declr1)))
-	 (if (eq? xdeclr declr1) declr `(param-declr ,xdeclr))))
-      ((array-of ,declr1 ,array-spec)
-       (let ((xdeclr (fix-declr declr1)))
-	 (if (eq? xdeclr declr1) declr `(array-of ,xdeclr ,array-spec))))
-      ((array-of ,dir-declr)
-       (let ((xdeclr (fix-declr dir-declr)))
-	 (if (eq? xdeclr dir-declr) declr `(array-of ,xdeclr))))
-      ((ptr-declr ,pointer ,dir-declr)
-       (let ((xdeclr (fix-declr dir-declr)))
-	 (if (eq? xdeclr dir-declr) declr `(ptr-declr ,pointer ,xdeclr))))
+    (and
+     declr
+     (sx-match declr
+       ((ident ,name) declr)
+       ((bit-field . ,rest) declr)
+       
+       ((init-declr ,declr1 . ,rest)
+	(let ((xdeclr (fix-declr declr1)))
+	  (if (eq? xdeclr declr1) declr `(init-declr ,xdeclr . ,rest))))
+       ((comp-declr ,declr1)
+	(let ((xdeclr (fix-declr declr1)))
+	  (if (eq? xdeclr declr1) declr `(comp-declr ,xdeclr))))
+       ((param-declr ,declr1)
+	(let ((xdeclr (fix-declr declr1)))
+	  (if (eq? xdeclr declr1) declr `(param-declr ,xdeclr))))
+       ((array-of ,declr1 ,array-spec)
+	(let ((xdeclr (fix-declr declr1)))
+	  (if (eq? xdeclr declr1) declr `(array-of ,xdeclr ,array-spec))))
+       ((array-of ,dir-declr)
+	(let ((xdeclr (fix-declr dir-declr)))
+	  (if (eq? xdeclr dir-declr) declr `(array-of ,xdeclr))))
+       ((ptr-declr ,pointer ,dir-declr)
+	(let ((xdeclr (fix-declr dir-declr)))
+	  (if (eq? xdeclr dir-declr) declr `(ptr-declr ,pointer ,xdeclr))))
 
-      ((scope ,declr1)
-       (let ((xdeclr (fix-declr declr1)))
-	 (if (eq? xdeclr declr1) declr `(scope ,xdeclr))))
+       ((scope ,declr1)
+	(let ((xdeclr (fix-declr declr1)))
+	  (if (eq? xdeclr declr1) declr `(scope ,xdeclr))))
 
-      ;; abstract declarator and direct abstract declarator
-      ((abs-declr ,pointer ,dir-abs-declr)
-       (let ((xdeclr (fix-declr dir-abs-declr)))
-	 (if (eq? xdeclr dir-abs-declr) declr `(abs-declr ,pointer ,xdeclr))))
-      ((abs-declr (pointer))
-       declr)
-      ((abs-declr (pointer ,pointer-val))
-       declr)
-      ((abs-declr ,dir-abs-declr)
-       (let ((xdeclr (fix-declr dir-abs-declr)))
-	 (if (eq? xdeclr dir-abs-declr) declr `(abs-declr ,xdeclr))))
+       ;; abstract declarator and direct abstract declarator
+       ((abs-declr ,pointer ,dir-abs-declr)
+	(let ((xdeclr (fix-declr dir-abs-declr)))
+	  (if (eq? xdeclr dir-abs-declr) declr `(abs-declr ,pointer ,xdeclr))))
+       ((abs-declr (pointer))
+	declr)
+       ((abs-declr (pointer ,pointer-val))
+	declr)
+       ((abs-declr ,dir-abs-declr)
+	(let ((xdeclr (fix-declr dir-abs-declr)))
+	  (if (eq? xdeclr dir-abs-declr) declr `(abs-declr ,xdeclr))))
 
-      ;; declr-scope
-      ;; declr-array dir-abs-declr
-      ;; declr-array dir-abs-declr assn-expr
-      ;; declr-array dir-abs-declr type-qual-list
-      ;; declr-array dir-abs-declr type-qual-list assn-expr
-      ((declr-scope ,abs-declr)		; ( abs-declr )
-       (let ((xdeclr (fix-declr abs-declr)))
-	 (if (eq? xdeclr abs-declr) declr `(declr-scope ,xdeclr))))
-      ((declr-array ,dir-abs-declr)	; []
-       (let ((xdeclr (fix-declr dir-abs-declr)))
-	 (if (eq? xdeclr dir-abs-declr) declr `(declr-array ,xdeclr))))
-      ((declr-array ,dir-abs-declr (type-qual-list . ,type-quals))
-       (let ((xdeclr (fix-declr dir-abs-declr)))
-	 (if (eq? xdeclr dir-abs-declr) declr
-	     `(declr-array ,xdeclr (type-qual-list . ,type-quals)))))
-      ((declr-array ,dir-abs-declr ,assn-expr)
-       (let ((xdeclr (fix-declr dir-abs-declr)))
-	 (if (eq? xdeclr dir-abs-declr) declr
-	     `(declr-array ,xdeclr ,assn-expr))))
-      ((declr-array ,dir-abs-declr ,type-qual-list ,assn-expr)
-       (let ((xdeclr (fix-declr dir-abs-declr)))
-	 (if (eq? xdeclr dir-abs-declr) declr
-	     `(declr-array ,xdeclr ,type-qual-list ,assn-expr))))
-      
-      ;; declr-anon-array type-qual-list assn-expr
-      ((declr-anon-array ,type-qual-list ,assn-expr) declr)
-      ;; declr-anon-array type-qual-list 
-      ((declr-anon-array (type-qual-list ,type-qual-tail)) declr)
-      ;; ???? declr-anon-array assn-expr
-      ((declr-anon-array ,assn-expr) declr)
-      ;; declr-anon-array
-      ((declr-anon-array) declr)
-      ;; declr-star dir-abs-decl
-      ((declr-star ,dir-abs-declr)
-       (let ((xdeclr (fix-declr dir-abs-declr)))
-	 (if (eq? xdeclr dir-abs-declr) declr
-	     `(declr-star ,xdeclr))))
-      ;; declr-star
-      ((declr-star) declr)
+       ;; declr-scope
+       ;; declr-array dir-abs-declr
+       ;; declr-array dir-abs-declr assn-expr
+       ;; declr-array dir-abs-declr type-qual-list
+       ;; declr-array dir-abs-declr type-qual-list assn-expr
+       ((declr-scope ,abs-declr)		; ( abs-declr )
+	(let ((xdeclr (fix-declr abs-declr)))
+	  (if (eq? xdeclr abs-declr) declr `(declr-scope ,xdeclr))))
+       ((declr-array ,dir-abs-declr)	; []
+	(let ((xdeclr (fix-declr dir-abs-declr)))
+	  (if (eq? xdeclr dir-abs-declr) declr `(declr-array ,xdeclr))))
+       ((declr-array ,dir-abs-declr (type-qual-list . ,type-quals))
+	(let ((xdeclr (fix-declr dir-abs-declr)))
+	  (if (eq? xdeclr dir-abs-declr) declr
+	      `(declr-array ,xdeclr (type-qual-list . ,type-quals)))))
+       ((declr-array ,dir-abs-declr ,assn-expr)
+	(let ((xdeclr (fix-declr dir-abs-declr)))
+	  (if (eq? xdeclr dir-abs-declr) declr
+	      `(declr-array ,xdeclr ,assn-expr))))
+       ((declr-array ,dir-abs-declr ,type-qual-list ,assn-expr)
+	(let ((xdeclr (fix-declr dir-abs-declr)))
+	  (if (eq? xdeclr dir-abs-declr) declr
+	      `(declr-array ,xdeclr ,type-qual-list ,assn-expr))))
+       
+       ;; declr-anon-array type-qual-list assn-expr
+       ((declr-anon-array ,type-qual-list ,assn-expr) declr)
+       ;; declr-anon-array type-qual-list 
+       ((declr-anon-array (type-qual-list ,type-qual-tail)) declr)
+       ;; ???? declr-anon-array assn-expr
+       ((declr-anon-array ,assn-expr) declr)
+       ;; declr-anon-array
+       ((declr-anon-array) declr)
+       ;; declr-star dir-abs-decl
+       ((declr-star ,dir-abs-declr)
+	(let ((xdeclr (fix-declr dir-abs-declr)))
+	  (if (eq? xdeclr dir-abs-declr) declr
+	      `(declr-star ,xdeclr))))
+       ;; declr-star
+       ((declr-star) declr)
 
-      ;; ftn-declr
-      ((ftn-declr ,dir-declr ,param-list)
-       (let ((xdeclr (fix-declr dir-declr))
-	     (xparam-list (fix-param-list param-list)))
-	 (if (and (eq? xdeclr dir-declr) (eq? xparam-list param-list)) declr
-	     `(ftn-declr ,xdeclr ,xparam-list))))
-      
-      ((abs-ftn-declr ,dir-declr ,param-list)
-       (let ((xdeclr (fix-declr dir-declr))
-	     (xparam-list (if (eq? 'param-list (sx-tag param-list))
-			      (fix-param-list param-list)
-			      param-list)))
-	 (if (and (eq? xdeclr dir-declr) (eq? xparam-list param-list)) declr
-	     `(abs-ftn-declr ,xdeclr ,xparam-list))))
-      ((anon-ftn-declr ,param-list)
-       (let ((xparam-list (fix-param-list param-list)))
-	 (if (eq? xparam-list param-list) declr
-	     `(anon-ftn-declr ,xparam-list))))
+       ;; ftn-declr
+       ((ftn-declr ,dir-declr ,param-list)
+	(let ((xdeclr (fix-declr dir-declr))
+	      (xparam-list (fix-param-list param-list)))
+	  (if (and (eq? xdeclr dir-declr) (eq? xparam-list param-list)) declr
+	      `(ftn-declr ,xdeclr ,xparam-list))))
+       
+       ((abs-ftn-declr ,dir-declr ,param-list)
+	(let ((xdeclr (fix-declr dir-declr))
+	      (xparam-list (if (eq? 'param-list (sx-tag param-list))
+			       (fix-param-list param-list)
+			       param-list)))
+	  (if (and (eq? xdeclr dir-declr) (eq? xparam-list param-list)) declr
+	      `(abs-ftn-declr ,xdeclr ,xparam-list))))
+       ((anon-ftn-declr ,param-list)
+	(let ((xparam-list (fix-param-list param-list)))
+	  (if (eq? xparam-list param-list) declr
+	      `(anon-ftn-declr ,xparam-list))))
 
-      ;; declr-lists too
-      ((init-declr-list . ,declrs)
-       (let ((xdeclrs (map fix-declr declrs)))
-	 (if (fold (lambda (l r seed) (and (eq? l r) seed)) #t xdeclrs declrs)
-	     declr
-	     `(init-declr-list . ,xdeclrs))))
-      ((comp-declr-list . ,declrs)
-       (let ((xdeclrs (map fix-declr declrs)))
-	 ;;(sferr "    declrs, xdeclrs:\n") (pperr declrs) (pperr xdeclrs)
-	 (if (fold (lambda (l r seed) (and (eq? l r) seed)) #t xdeclrs declrs)
-	     declr
-	     `(comp-declr-list . ,xdeclrs))))
-      
-      (,otherwise (throw 'util-error "c99/munge: unknown declarator: " declr))))
+       ;; declr-lists too
+       ((init-declr-list . ,declrs)
+	(let ((xdeclrs (map fix-declr declrs)))
+	  (if (fold (lambda (l r seed) (and (eq? l r) seed)) #t xdeclrs declrs)
+	      declr
+	      `(init-declr-list . ,xdeclrs))))
+       ((comp-declr-list . ,declrs)
+	(let ((xdeclrs (map fix-declr declrs)))
+	  ;;(sferr "    declrs, xdeclrs:\n") (pperr declrs) (pperr xdeclrs)
+	  (if (fold (lambda (l r seed) (and (eq? l r) seed)) #t xdeclrs declrs)
+	      declr
+	      `(comp-declr-list . ,xdeclrs))))
+       
+       (* (throw 'util-error "c99/munge: unknown declarator: " declr)))))
 
   (let*-values (((tag attr orig-specl orig-declr tail)
 		 (split-adecl adecl))
@@ -1106,10 +1109,10 @@
 	 ;; else ...
 	 seed))
    seed udict))
- 
+
 
 ;; === enum handling ===================
-  
+
 ;; @deffn {Procedure} canize-enum-def-list enum-def-list [defs] => enum-def-list
 ;; Fill in constants for all entries of an enum list.
 ;; Expects @code{(enum-def-list (...))} (i.e., not the tail).
@@ -1129,7 +1132,7 @@
      ((null? edl)
       (sx-cons* (sx-tag enum-def-list) (sx-attr enum-def-list) (reverse rez)))
      (else
-      (sxml-match (car edl)
+      (sx-match (car edl)
 	((enum-defn ,ident)
 	 (let ((sval (number->string nxt)))
 	   (iter (cons `(enum-defn ,ident (fixed ,sval)) rez)
@@ -1205,10 +1208,10 @@
 	  (cons (reverse kseed) seed)
 	  seed))
      (else (cons (reverse kseed) seed))))
-	
+  
   (define (fH seed tree)
     (cons tree seed))
-   
+  
   (foldts fD fU fH '() declr))
 
 ;; @deffn {Procedure} stripdown udecl => udecl
@@ -1337,25 +1340,24 @@
 
   (define (unwrap-pointer pointer)  ;; =>list IGNORES TYPE QUALIFIERS
     ;;(sferr "unwrap-pointer ~S\n" pointer)
-    (sxml-match pointer
-      ((pointer (type-qual-list ,type-qual ...) ,pointer)
+    (sx-match pointer
+      ((pointer (type-qual-list . ,type-qual) ,pointer)
        (cons '(pointer-to) (unwrap-pointer pointer)))
-      ((pointer (type-qual-list ,type-qual ...)) '((pointer-to)))
+      ((pointer (type-qual-list . ,type-qual)) '((pointer-to)))
       ((pointer ,pointer) (cons '(pointer-to) (unwrap-pointer pointer)))
       ((pointer) '((pointer-to)))
-      (,otherwise
+      (*
        (sferr "unwrap-pointer failed on:\n") (pperr pointer)
-       (error "unwrap-pointer"))
-      ))
+       (error "unwrap-pointer"))))
 
   (define (make-abs-dummy) ;; for abstract declarator make a dummy
     (or abs-ident (symbol->string (gensym "@"))))
   (define (make-abs-dummy-tail)
     (list (make-abs-dummy)))
-    
+  
   (define* (unwrap-declr declr #:key (const #f))
     ;;(sferr "unwrap-declr:\n") (pperr declr #:per-line-prefix "  ")
-    (sxml-match declr
+    (sx-match declr
       ((ident ,name)
        (list name))
       ((init-declr ,item)
@@ -1414,9 +1416,9 @@
       ((comp-declr ,item) (unwrap-declr item))
       ((param-declr ,item) (unwrap-declr item))
 
-      (,otherwise
+      (*
        (sferr "munge/unwrap-declr missed:\n")
-       (pperr otherwise)
+       (pperr declr)
        (error "c99/munge: udecl->mspec failed")
        #f)))
 
@@ -1460,10 +1462,10 @@
       
       (((pointer-to) . ,rest)
        (doit `(ptr-declr (pointer) ,declr) rest))
-       
-      (,otherwise
+      
+      (,*
        (sferr "munge/mspec->udecl missed:\n")
-       (pperr otherwise)
+       (pperr mspec-tail)
        (error "munge/mspec->udecl failed")
        #f)))
 
@@ -1511,7 +1513,7 @@
      ((eqv? (sx-tag tree) 'type-qual) seed)
      ((null? seed) (reverse kseed))
      (else (cons (reverse kseed) seed))))
-	
+  
   (define (fsH seed tree)
     (cons tree seed))
 
@@ -1525,40 +1527,3 @@
 
 
 ;; --- last line ---
-#|
-;; @deffn {Procedure} splice-declarators orig-declr tdef-declr => 
-;; Splice the original declarator into the typedef declarator.
-;; This is a helper for @code{expand-*-typename-ref} procecures.
-;; Consider
-;; @example
-;; typedef int *foo_t;
-;; foo_t bla[3];
-;; @end example
-;; @noindent
-;; maps
-;; @example
-;; bla[3] => *(bla[3])
-;; @end example
-;; @end deffn
-(define (splice-declarators orig-declr tdef-declr)
-  
-  (define (fD seed tree)		; => (values seed tree)
-    (sxml-match tree
-      ((param-list . ,rest) (values tree '())) ; don't process
-      ((ident ,name) (values (reverse (cadr orig-declr)) '())) ; replace
-      (,otherwise (values '() tree))))
-
-  (define (fU seed kseed tree)
-    (let ((ktree (case (car kseed)
-		   ((param-list ident) kseed)
-		   (else (reverse kseed)))))
-      (if (null? seed) ktree (cons ktree seed))))
-
-  (define (fH seed tree)
-    (cons tree seed))
-
-  ;; This cons transfers the tag from orig-declr to the result.
-  (cons
-   (car orig-declr)			; init-declr or comp-declr
-   (cdr (foldts* fD fU fH '() tdef-declr)))) ; always init-declr
-|#
