@@ -1,19 +1,19 @@
-;; nyacc/lang/c99/parser.scm
-;;
-;; Copyright (C) 2015-2017 Matthew R. Wette
-;;
-;; This program is free software: you can redistribute it and/or modify
-;; it under the terms of the GNU General Public License as published by 
-;; the Free Software Foundation, either version 3 of the License, or 
-;; (at your option) any later version.
-;;
-;; This program is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;; GNU General Public License for more details.
-;;
-;; You should have received a copy of the GNU General Public License
-;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;;; nyacc/lang/c99/parser.scm
+;;;
+;;; Copyright (C) 2015-2018 Matthew R. Wette
+;;;
+;;; This library is free software; you can redistribute it and/or
+;;; modify it under the terms of the GNU Lesser General Public
+;;; License as published by the Free Software Foundation; either
+;;; version 3 of the License, or (at your option) any later version.
+;;;
+;;; This library is distributed in the hope that it will be useful,
+;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;;; Lesser General Public License for more details.
+;;;
+;;; You should have received a copy of the GNU Lesser General Public License
+;;; along with this library; if not, see <http://www.gnu.org/licenses/>.
 
 ;; C parser
 
@@ -24,6 +24,13 @@
   #:use-module (nyacc lang util)
   #:use-module (nyacc lang c99 cpp)
   )
+(cond-expand
+  (guile-2 #t)
+  (else
+   (use-modules (srfi srfi-16))
+   (use-modules (ice-9 optargs))
+   (use-modules (ice-9 syncase))
+   (use-modules (nyacc compat18))))
 
 (include-from-path "nyacc/lang/c99/mach.d/c99tab.scm")
 (include-from-path "nyacc/lang/c99/body.scm")
