@@ -1,27 +1,31 @@
 ;; nyacc/lang/c99/pprint.scm
+
+;; Copyright (C) 2015-2018 Matthew R. Wette
 ;;
-;; Copyright (C) 2015-2017 Matthew R. Wette
+;; This library is free software; you can redistribute it and/or
+;; modify it under the terms of the GNU Lesser General Public
+;; License as published by the Free Software Foundation; either
+;; version 3 of the License, or (at your option) any later version.
 ;;
-;; This program is free software: you can redistribute it and/or modify
-;; it under the terms of the GNU General Public License as published by 
-;; the Free Software Foundation, either version 3 of the License, or 
-;; (at your option) any later version.
+;; This library is distributed in the hope that it will be useful,
+;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;; Lesser General Public License for more details.
 ;;
-;; This program is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;; GNU General Public License for more details.
-;;
-;; You should have received a copy of the GNU General Public License
-;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;; You should have received a copy of the GNU Lesser General Public License
+;; along with this library; if not, see <http://www.gnu.org/licenses/>
 
 (define-module (nyacc lang c99 pprint)
   #:export (pretty-print-c99)
   #:use-module ((srfi srfi-1) #:select (pair-for-each))
   #:use-module (nyacc lang util)
-  #:use-module (sxml match)
+  #:use-module (nyacc lang sx-match)
   #:use-module (ice-9 pretty-print)
   )
+(cond-expand
+ (guile-2 #t)
+ (else
+  (use-modules (ice-9 optargs))))
 
 (define op-sym
   (let ((ot '(("=" . eq) ("+=" . pl-eq) ("-=" . mi-eq) ("*=" . ti-eq)
@@ -67,14 +71,8 @@
 (define (scmchs->c scm-chr-str)
   (let ((ch (string-ref scm-chr-str 0)))
     (case ch
-      ((#\nul) "\\0")
-      ((#\alarm) "\\a")
-      ((#\backspace) "\\b")
-      ((#\tab) "\\t")
-      ((#\newline) "\\n")
-      ((#\vtab) "\\v")
-      ((#\page) "\\f")
-      ((#\\) "\\")
+      ((#\nul) "\\0") ((#\bel) "\\a") ((#\bs) "\\b") ((#\ht) "\\t")
+      ((#\nl) "\\n") ((#\vt) "\\v") ((#\np) "\\f") ((#\cr) "\\r") ((#\\) "\\")
       (else scm-chr-str))))
 
 ;; @deffn {Procedure} char->hex-list ch seed
@@ -89,6 +87,10 @@
      (else
       (iter (cons (itox (remainder val 16)) res) (1- ix) (quotient val 16))))))
 
+(define (esc->ch ch)
+  (case ch ((#\nul) #\0) ((#\bel) #\a) ((#\bs) #\b) ((#\ht) #\t)
+	((#\nl) #\n) ((#\vt) #\v) ((#\np) #\f) ((#\cr) #\r)))
+   
 ;; @deffn {Procedure} scmstr->c str
 ;; to be documented
 ;; @end deffn
@@ -98,7 +100,9 @@
     (lambda (ch chl)
       (cond
        ((char-set-contains? char-set:printing ch) (cons ch chl))
-       ((char=? ch #\nul) (cons* #\\ #\0 chl))
+       ((char=? ch #\space) (cons #\space chl))
+       ((memq ch '(#\nul #\bel #\bs #\ht #\nl #\vt #\np #\cr))
+	(cons* #\\ (esc->ch ch) chl))
        (else (char->hex-list ch chl))))
     '() str)))
     
@@ -131,7 +135,7 @@
 
   (define (cpp-ppx tree)
     (fmtr 'nlin)
-    (sxml-match tree
+    (sx-match tree
       ((define (name ,name) (args . ,args) (repl ,repl))
        (sf "#define ~A(" name)
        (pair-for-each
@@ -146,11 +150,10 @@
       ((else) (sf "#else\n"))
       ((endif ,text) (sf "#endif ~A\n" text))
       ((endif) (sf "#endif\n"))
-      ((include . ,rest) (sf "#include ~A\n" (sx-ref tree 1)))
+      ((include ,file . *) (sf "#include ~A\n" file))
       ((error ,text) (sf "#error ~A\n" text))
       ((pragma ,text) (sf "#pragma ~A\n" text))
-      (,otherwise
-       (simple-format #t "\n*** pprint/cpp-ppx: NO MATCH: ~S\n" tree)))
+      (* (simple-format #t "\n*** pprint/cpp-ppx: NO MATCH: ~S\n" tree)))
     (fmtr 'nlin))
 
   (define protect-expr? (make-protect-expr op-prec op-assc))
@@ -176,21 +179,6 @@
 	(ppx/p rval)
 	(ppx rval)))
 
-  (define (comp declr initr)
-    (let ((iexpr (and initr (sx-ref initr 1))))
-      (ppx declr)
-      (when initr
-	(sf " = ")
-	(case (sx-tag iexpr)
-	  ((initzer-list)
-	   (sf "{")
-	   (for-each
-	    (lambda (expr) (ppx (sx-ref expr 1)) (sf ", "))
-	    (sx-tail iexpr 1))
-	   (sf "}"))
-	  (else
-	   (ppx iexpr))))))
-
   (define (struct-union-def struct-or-union name fields)
     (if name
 	(sf "~A ~A {\n" struct-or-union name)
@@ -204,18 +192,25 @@
 
   ;; TODO: comp-lit
   (define (ppx-1 tree)
-    (sxml-match tree
+    (sx-match tree
 
       ((p-expr ,expr) (ppx expr))
       ((ident ,name) (sf "~A" name))
-      ((char ,value) (sf "'~A'" (scmchs->c (sx-ref tree 1))))
+      ((char (@ . ,al) ,value)
+       (let ((type (sx-attr-ref al 'type)))
+	 (cond
+	  ((not type) (sf "'~A'" (scmchs->c value)))
+	  ((string=? type "wchar_t") (sf "L'~A'" (scmchs->c value)))
+	  ((string=? type "char16_t") (sf "u'~A'" (scmchs->c value)))
+	  ((string=? type "char32_t") (sf "U'~A'" (scmchs->c value)))
+	  (else (throw 'c99-error "bad type")))))
       ((fixed ,value) (sf "~A" value))
       ((float ,value) (sf "~A" value))
 
       ((string . ,value-l)
        (pair-for-each
 	(lambda (pair)
-	  (sf "~S" (scmstr->c (car pair)))
+	  (sf "\"~A\"" (scmstr->c (car pair)))
 	  (if (pair? (cdr pair)) (sf " ")))
 	value-l))
 
@@ -228,7 +223,8 @@
        ;; needed:
        ;; 1) (get-col) to get column
        ;; 2) (indent-to-col col) to do indents for each line
-       #;(let ((col (get-col))
+       #!
+       (let ((col (get-col))
 	     (lines (string-split text #\newline))
 	     (ind (mk-ind-to-col col)))
 	 (sf "/*") (sf (car lines))
@@ -239,6 +235,7 @@
 	    (sf (cdr pair)))
 	  (cdr lines))
 	 (sf "*/"))
+       !#
        )
       
       ((scope ,expr) (sf "(") (ppx expr) (sf ")"))
@@ -329,18 +326,6 @@
 	   (else (ppx (car pair)))))
 	expr-list))
 
-      ;; #|
-      ;; gotta break up ppx because sxml-match seems to eat stack space:
-      ;; everthing together results in SIGABRT from vm_error_stack_overflow()
-      (,otherwise
-       (ppx-2 tree))))
-  
-  (define (ppx-2 tree)
-    
-    (sxml-match tree
-      ;; sxml-match continues here to avoid stack overflow
-      ;; |#
-
       ((udecl . ,rest)
        (ppx `(decl . ,rest)))
       ((decl ,decl-spec-list)
@@ -388,10 +373,12 @@
 	  (if (pair? (cdr pair)) (sf ", ")))
 	rest))
 
-      ((init-declr ,declr ,initr) (comp declr initr))
-      ((init-declr ,declr) (comp declr #f))
-      ((comp-declr ,declr) (comp declr #f))
-      ((param-declr ,declr) (comp declr #f))
+      ((init-declr ,declr ,initr) (ppx declr) (ppx initr))
+      ((init-declr ,declr) (ppx declr))
+      ((comp-declr ,declr) (ppx declr))
+      ((param-declr ,declr) (ppx declr))
+      ;; This should work with sx-match, to replace above three.
+      ;;(((init-declr comp-declr param-declr) ,declr) (ppx declr))
 
       ((bit-field ,ident ,expr)
        (ppx ident) (sf " : ") (ppx expr))
@@ -445,8 +432,9 @@
       ((enum-defn (ident ,name))
        (sf "~A,\n" name))
 
-      ((fctn-spec "inline")
-       (sf "inline "))			; SPACE ???
+      ;;((fctn-spec "inline")
+      ((fctn-spec ,spec)
+       (sf "~S " spec))			; SPACE ???
 
       ((ptr-declr ,ptr ,dir-declr)
        (ppx ptr) (ppx dir-declr))
@@ -504,23 +492,25 @@
        (ppx dir-abs-declr) (sf "(") (ppx param-type-list) (sf ")"))
       ;; anon-ftn-declr
 
+      ;; initializer
+      ((initzer ,expr)
+       (sf " = ") (ppx expr))
+      
+      ;; initializer-list
+      ((initzer-list . ,items)
+       (sf "{") ;; or "{ "
+       (pair-for-each
+	(lambda (pair)
+	  (ppx (sx-ref (car pair) 1))
+	  (if (pair? (cdr pair)) (sf ", ")))
+	items)
+       (sf "}")) ;; or " }"
+
       ((compd-stmt (block-item-list . ,items))
        (sf "{\n") (push-il) (for-each ppx items) (pop-il) (sf "}\n"))
       ((compd-stmt-no-newline (block-item-list . ,items))
        (sf "{\n") (push-il) (for-each ppx items) (pop-il) (sf "} "))
       
-      ;; #|
-      ;; gotta break up ppx because sxml-match seems to eat stack space:
-      ;; everthing together results in SIGABRT from vm_error_stack_overflow()
-      (,otherwise
-       (ppx-3 tree))))
-  
-  (define (ppx-3 tree)
-
-    (sxml-match tree
-      ;; sxml-match continues here to avoid stack overflow
-      ;; |#
-      
       ;; expression-statement
       ((expr-stmt) (sf ";\n"))
       ((expr-stmt ,expr) (ppx expr) (sf ";\n"))
@@ -654,9 +644,7 @@
       ((extern-begin ,lang) (sf "extern \"~A\" {\n" lang))
       ((extern-end) (sf "}\n"))
 
-      (,otherwise
-       (simple-format #t "\n*** pprint/ppx: NO MATCH: ~S\n" (car tree)))
-      ))
+      (* (simple-format #t "\n*** pprint/ppx: NO MATCH: ~S\n" (car tree)))))
 
   (define ppx ppx-1)
 
