@@ -1,19 +1,19 @@
-;; lang/c/util1.scm
-;;
-;; Copyright (C) 2015-2017 Matthew R. Wette
-;;
-;; This program is free software: you can redistribute it and/or modify
-;; it under the terms of the GNU General Public License as published by 
-;; the Free Software Foundation, either version 3 of the License, or 
-;; (at your option) any later version.
-;;
-;; This program is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of 
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;; GNU General Public License for more details.
-;;
-; You should have received a copy of the GNU General Public License
-;; along with this program.  If not, see <http://www.gnu.org/licenses/>.
+;;; lang/c/util1.scm
+;;;
+;;; Copyright (C) 2015-2018 Matthew R. Wette
+;;;
+;;; This library is free software; you can redistribute it and/or
+;;; modify it under the terms of the GNU Lesser General Public
+;;; License as published by the Free Software Foundation; either
+;;; version 3 of the License, or (at your option) any later version.
+;;;
+;;; This library is distributed in the hope that it will be useful,
+;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;;; Lesser General Public License for more details.
+;;;
+;;; You should have received a copy of the GNU Lesser General Public License
+;;; along with this library; if not, see <http://www.gnu.org/licenses/>.
 
 ;; C parser utilities
 
@@ -24,10 +24,11 @@
 	    merge-inc-trees!
 	    elifify)
   #:use-module (nyacc lang util)
+  #:use-module (nyacc lang sx-match)
   #:use-module ((srfi srfi-1) #:select (append-reverse))
   #:use-module (srfi srfi-2) ;; and-let*
   #:use-module (sxml fold)
-  #:use-module (sxml match)
+  ;;#:use-module (sxml match) ; use sx-match instead
   #:use-module (ice-9 popen)		; gen-cc-defs
   #:use-module (ice-9 rdelim)		; gen-cc-defs
   #:use-module (ice-9 regex)		; gen-cc-defs
@@ -97,34 +98,25 @@
 	   (if (eof-object? line) lines
 	       (iter (cons line lines) (read-line ip 'trim)))))))))
 
-;; @deffn {Procedure} get-gcc-inc-dirs [args] [#:CC "gcc"] => '("ABC=123" ...)
+;; @deffn {Procedure} get-gcc-inc-dirs [args] [#:CC "gcc"] =>
 ;; Generate a list of compiler-internal include directories (for gcc).  If
 ;; keyword arg @arg{CC} is not provided this procedure looks for environment
 ;; variable @code{"CC"}, else it defaults to @code{"gcc"}.
 ;; @end deffn
-(define get-gcc-inc-dirs
-  (let ((rx1 (make-regexp "(.*): *=(.*)$"))
-	(rx2 (make-regexp "(.*): *(.*)$")))
-    (lambda* (#:optional (args '()) #:key CC)
-      (let* ((ip (open-input-pipe (string-append (resolve-CC CC)
-						" -print-search-dirs")))
-	     (il (let iter ((items '()) (line (read-line ip 'trim)))
-		   (cond
-		    ((eof-object? line)
-		     items)
-		    (else
-		     (let* ((m1 (regexp-exec rx1 line))
-			    (m2 (regexp-exec rx2 line)))
-		       (cond
-			(m1 (acons (match:substring m1 1)
-				   (string-split #\: (match:substring m1 2))
-				   items))
-			(m2 (acons (match:substring m2 1)
-				   (match:substring m2 2)
-				   items))
-			(else (sferr "get-gcc-inc-dirs: bad match\n")
-			      items))))))))
-	(list (string-append (assoc-ref il "install") "/include"))))))
+(define* (get-gcc-inc-dirs #:optional (args '()) #:key CC)
+  (let ((ip (open-input-pipe (string-append
+			      (resolve-CC CC) " -E -Wp,-v - </dev/null 2>&1"))))
+    (let iter ((dirs '()) (grab #f) (line (read-line ip 'trim)))
+      (cond
+       ((eof-object? line) dirs)
+       ((string=? line "#include <...> search starts here:")
+	(iter dirs #t (read-line ip 'trim)))
+       ((string=? line "End of search list.") dirs)
+       (grab
+	(iter (cons (string-trim-both line) dirs)
+	      grab (read-line ip 'trim)))
+       (else
+	(iter dirs grab (read-line ip 'trim)))))))
 
 ;; @deffn {Procedure} remove-inc-trees tree
 ;; Remove the trees included with cpp-include statements.
@@ -227,13 +219,14 @@
 ;; @end deffn
 (define (elifify tree)
   (define (fU tree)
-    (sxml-match tree
+    (sx-match tree
       ((if ,x1 ,t1 (if ,x2 ,t2 (else-if ,x3 ,t3) . ,rest))
        `(if ,x1 ,t1 (else-if ,x2 ,t2) (else-if ,x3 ,t3) . ,rest))
       ((if ,x1 ,t1 (if ,x2 ,t2 . ,rest))
        `(if ,x1 ,t1 (else-if ,x2 ,t2) . ,rest))
-      (,otherwise
-       tree)))
+      (*
+       tree))
+    )
   (foldt fU identity tree))
        
 ;; --- last line ---
