@@ -1,10 +1,19 @@
 ;;; module/nyacc/util.scm
 ;;;
-;;; Copyright (C) 2015-2017 Matthew R. Wette
+;;; Copyright (C) 2015-2018 Matthew R. Wette
 ;;;
-;;; This software is covered by the GNU GENERAL PUBLIC LICENCE, Version 3,
-;;; or any later version published by the Free Software Foundation.  See
-;;; the file COPYING included with the nyacc distribution.
+;;; This library is free software; you can redistribute it and/or
+;;; modify it under the terms of the GNU Lesser General Public
+;;; License as published by the Free Software Foundation; either
+;;; version 3 of the License, or (at your option) any later version.
+;;;
+;;; This library is distributed in the hope that it will be useful,
+;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
+;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;;; Lesser General Public License for more details.
+;;;
+;;; You should have received a copy of the GNU Lesser General Public License
+;;; along with this library; if not, see <http://www.gnu.org/licenses/>.
 
 ;; runtime utilities for the parsers
 
@@ -30,14 +39,22 @@
   ;; #:use-module ((sxml xpath) #:select (sxpath)) ;; see sx-find below
   #:use-module (ice-9 pretty-print)
   )
+(cond-expand
+  (guile-2
+   #t)
+  (guile
+   (use-modules (ice-9 optargs))
+   (use-modules (srfi srfi-16))))
 
 ;; This is a generic copyright/licence that will be printed in the output
 ;; of the examples/nyacc/lang/*/ actions.scm and tables.scm files.
 (define lang-crn-lic "
 
-This software is covered by the GNU GENERAL PUBLIC LICENCE, Version 3,
-or any later version published by the Free Software Foundation.  See
-the file COPYING included with the this distribution.")
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 3 of the License, or (at your option) any later version.
+See the file COPYING.LESSER included with the this distribution.")
 
 (define (sferr fmt . args)
   (apply simple-format (current-error-port) fmt args))
@@ -54,7 +71,7 @@ the file COPYING included with the this distribution.")
 
 ;; === input stack =====================
 
-(define *input-stack* (make-fluid #f))
+(define *input-stack* (make-fluid))
 
 (define (reset-input-stack)
   (fluid-set! *input-stack* '()))
@@ -314,13 +331,17 @@ the file COPYING included with the this distribution.")
 	  #f)
       #f))
 
-;; @deffn {Procedure} sx-attr-ref sx key => val
+;; @deffn {Procedure} sx-attr-ref sx|node|tail key => val
 ;; Return an attribute value given the key, or @code{#f}.
+;; Also works if passed the attribute node @code{(@ ...)} or its tail.
 ;; @end deffn
 (define (sx-attr-ref sx key)
-  (and=> (sx-attr sx)
-	 (lambda (attr) (and=> (assq-ref (cdr attr) key)
-			       car))))
+  (let ((attr-tail (cond ((null? sx) sx)
+			 ((pair? (car sx)) sx)
+			 ((eqv? '@ (car sx)) (car sx))
+			 ((sx-attr sx))
+			 (else '()))))
+    (and=> (assq-ref attr-tail key) car)))
 
 ;; @deffn {Procedure} sx-attr-set! sx key val
 ;; Set attribute for sx.  If no attributes exist, if key does not exist,
@@ -372,9 +393,6 @@ the file COPYING included with the this distribution.")
     (find (lambda (node)
 	    (and (pair? node) (eqv? tag-or-path (car node))))
 	  sx))
-   #;((pair? tag-or-path)
-    (let ((rez ((sxpath tag-or-path) sx)))
-      (if (pair? rez) (car rez) #f)))
    (else
     (error "expecting first arg to be tag or sxpath"))))
 
