@@ -17,7 +17,7 @@
 
 ;; A module providing procedures for constructing lexical analyzers.
 
-;; '$fixed '$float '$string '$chlit '$ident
+;; '$fixed '$float '$string '$ident '$chlit '$chlit/L '$chlit/u '$chlit/U
 
 ;; todo: change lexer to return @code{cons-source} instead of @code{cons}
 ;; todo: to be fully compliant, C readers need to deal with \ at end of line
@@ -54,9 +54,15 @@
 	    make-like-ident-p
 	    )
   #:use-module ((srfi srfi-1) #:select (remove append-reverse))
-  #:use-module (ice-9 pretty-print)
   )
+(cond-expand
+  (guile-2 #t)
+  (guile
+   (use-modules (ice-9 optargs))
+   (use-modules (ice-9 syncase))))
 
+(define (sf fmt . args) (apply simple-format #t fmt args))
+  
 ;; @section Constructing Lexical Analyzers
 ;; The @code{lex} module provides a set of procedures to build lexical
 ;; analyzers.  The approach is to first build a set of @defn{readers} for 
@@ -186,24 +192,24 @@
 		(else (iter (cons ch cl) (read-char)))))
 	#f)))
 
-;; @deffn {Procedure} read-oct ch => "0123"|#f
-;; Read octal number.
+;; @deffn {Procedure} read-oct => 123|#f
+;; Read octal number, assuming @code{\0} have already been read.
+;; Return integer.
 ;; @end deffn
 (define read-oct
   (let ((cs:oct (string->char-set "01234567")))
-    (lambda (ch)
-      (let iter ((cv 0) (ch ch) (n 1))
+    (lambda ()
+      (let iter ((cv 0) (ch (read-char)) (n 0))
 	(cond
 	 ((eof-object? ch) cv)
-	 ((> n 3) (unread-char ch) cv)
+	 ;;((> n 3) (unread-char ch) cv)
 	 ((char-set-contains? cs:oct ch)
 	  (iter (+ (* 8 cv) (- (char->integer ch) 48)) (read-char) (1+ n)))
-	 (else
-	  (unread-char ch)
-	  cv))))))
+	 (else (unread-char ch) cv))))))
 
-;; @deffn {Procedure} read-hex => "0x7f"|#f
-;; Read octal number.  Assumes prefix (e.g., "0x" has already been read).
+;; @deffn {Procedure} read-hex => 123|#f
+;; Read hex number.  Assumes prefix (e.g., "0x" has already been read).
+;; Returns integer.
 ;; @end deffn
 (define read-hex
   (let ((cs:dig (string->char-set "0123456789"))
@@ -213,7 +219,7 @@
       (let iter ((cv 0) (ch (read-char)) (n 0))
 	(cond
 	 ((eof-object? ch) cv)
-	 ((> n 2) (unread-char ch) cv)
+	 ;;((> n 2) (unread-char ch) cv)
 	 ((char-set-contains? cs:dig ch)
 	  (iter (+ (* 16 cv) (- (char->integer ch) 48)) (read-char) (1+ n)))
 	 ((char-set-contains? cs:uhx ch)
@@ -236,16 +242,16 @@
       ((#\') (cons #\' seed))
       ((#\n) (cons #\newline seed))
       ((#\r) (cons #\return seed))
-      ((#\b) (cons #\backspace seed))
+      ((#\b) (cons #\bs seed))
       ((#\t) (cons #\tab seed))
       ((#\f) (cons #\page seed))
-      ((#\a) (cons #\alarm seed))
-      ((#\v) (cons #\vtab seed))
+      ((#\a) (cons #\bel seed))	      ; guile 1.8 doesn't know #\alarm
+      ((#\v) (cons #\vt seed))	      ; guile 1.8 doesn't know #\vtab
+      ((#\0) (cons (integer->char (read-oct)) seed))
+      ((#\1 #\2 #\3 #\4 #\5 #\6 #\7)
+       (unread-char ch) (cons (integer->char (read-oct)) seed))
       ((#\x) (cons (integer->char (read-hex)) seed))
-      (else
-       (if (char-numeric? ch)
-	   (cons (integer->char (read-oct ch)) seed)
-	   (cons ch seed))))))
+      (else (cons ch seed)))))
 
 ;; @deffn {Procedure} read-c-string ch => ($string . "foo")
 ;; Read a C-code string.  Output to code is @code{write} not @code{display}.
@@ -269,43 +275,50 @@
 
 ;; @deffn {Procedure} read-c-chlit ch
 ;; @example
-;; ... 'c' ... => (read-c-chlit #\') => '($ch-lit . #\c)
+;; ... 'c' ... => (read-c-chlit #\') => '($chlit . #\c)
 ;; @end example
+;; This will return @code{$chlit}, $code{$chlit/L} for @code{wchar_t},
+;; @code{$chlit/u} for @code{char16_t}, or @code{$chlit/U} for @code{char32_t}.
 ;; @end deffn
 (define (read-c-chlit ch)
+  (define (read-esc-char)
+    (let ((c2 (read-char)))
+      (case c2
+	((#\t) "\t")		   ; horizontal tab U+0009
+	((#\n) "\n")		   ; newline U+000A
+	((#\v) "\v")		   ; verticle tab U+000B
+	((#\f) "\f")		   ; formfeed U+000C
+	((#\r) "\r")		   ; return U+000D
+	((#\a) "\x07")		   ; alert U+0007
+	((#\b) "\x08")		   ; backspace U+0008 not in guile 1.8
+	((#\0) (string (integer->char (read-oct)))) ; octal
+	((#\1 #\2 #\3 #\4 #\5 #\6 #\7)		    ; octal
+	 (unread-char c2) (string (integer->char (read-oct))))
+	((#\x) (string (integer->char (read-hex)))) ; hex
+	((#\\ #\' #\" #\? #\|) (string c2))
+	(else (error "bad escape sequence" c2)))))
+  (define (wchar t)
+    (case t ((#\L) '$chlit/L) ((#\u) '$chlit/u) ((#\U) '$chlit/U)))
   (cond
    ((char=? ch #\')
-    (let ((c1 (read-char)) (c2 (read-char)))
-      (if (eqv? c1 #\\)
-	  (let ((c3 (read-char)))
-	    (cons '$chlit
-		  (case c2
-		    ((#\0) "\0")	   ; nul U+0000 (#\U+...)
-		    ((#\a) "\a")	   ; alert U+0007
-		    ((#\b) "\b")	   ; backspace U+0008
-		    ((#\t) "\t")	   ; horizontal tab U+0009
-		    ((#\n) "\n")	   ; newline U+000A
-		    ((#\v) "\v")	   ; verticle tab U+000B
-		    ((#\f) "\f")	   ; formfeed U+000C
-		    ((#\r) "\r")	   ; return U+000D
-		    ((#\\ #\' #\" #\? #\|) (string c2))
-		    (else (error "bad escape sequence")))))
-	  (cons '$chlit (string c1)))))
+    (let* ((c1 (read-char))
+	   (sc (if (eqv? c1 #\\) (read-esc-char) (string c1))))
+      (if (not (char=? #\' (read-char))) (error "bad char lit"))
+      (cons '$chlit sc)))
    ((char-set-contains? c:cx ch)
     (let ((c1 (read-char)))
       (cond
-       ((char=? c1 #\') (read-c-chlit c1))
+       ((char=? c1 #\') (cons (wchar ch) (cdr (read-c-chlit c1))))
        (else (unread-char c1) #f))))
    (else #f)))
 
 (define (fix-dot l) (if (char=? #\. (car l)) (cons #\0 l) l))
 
-;; @deffn {Procedure} make-num-reader => (proc ch) => #f|($fixed . "1")|($float . "1.0")
-;; Reads C numbers.
-;; This routine will clean by adding "0" before or after dot.
-;; may want to replace "eEdD" w/ "e"
-;; integer decimal(#t/#f) fraction exponent looking-at
-;; i, f and e are lists of characters
+;; @deffn {Procedure} make-num-reader => (proc ch) => #f|fixed|float
+;; Where @emph{fixed} is @code{($fixed . "1")} and @emph{float} is
+;; @code{($float . "1.0")}
+;; This procedure reads C numeric literals.
+;; Some literals are cleaned: @code{"0"} may be added before or after dot.
 ;; @end deffn
 (define (make-num-reader)
   ;; This will incorrectly parse 123LUL.  Does not handler hex floats.
