@@ -12,9 +12,8 @@
 ;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 ;;; Lesser General Public License for more details.
 ;;;
-;;; You should have received a copy of the GNU Lesser General Public
-;;; License along with this library; if not, write to the Free Software
-;;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+;;; You should have received a copy of the GNU Lesser General Public License
+;;; along with this library; if not, see <http://www.gnu.org/licenses/>
 
 (define-module (nyacc util)
   #:export (
@@ -28,7 +27,13 @@
 	    ugly-print OLD-ugly-print
 	    tzort
 	    )
-  #:use-module ((srfi srfi-43) #:select (vector-fold))
+  )
+(cond-expand
+  (guile-2
+   (use-modules (srfi srfi-43)))
+  (guile
+   (use-modules (ice-9 optargs))
+   (use-modules (nyacc compat18)))
   )
 
 (define (fmtstr fmt . args)
@@ -269,7 +274,8 @@
 	      (lambda (ix col elt)
 		(iter1 (if (zero? ix) col (strout col " ")) elt))
 	      (strout col "#(") sx) ")"))
-	   ((null? sx) (strout col "'()"))
+	   ;;((null? sx) (strout col "'()"))
+	   ((null? sx) (strout col "()"))
 	   (else (strout col (obj->str sx))))))
        (iter2
 	(lambda (col sx)
